@@ -39,8 +39,6 @@
 #endif
 #endif
 
-#ifndef NOXML
-
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -55,7 +53,9 @@
 #include <naf/nafrpc.h>
 #include <naf/nafhttpd.h>
 
-#include <libmx.h> /* Sadly, the entire point is XML. */
+#ifndef NOXML
+#include <libmx.h>
+#endif
 
 #include "module.h" /* for naf_module__registerresident() only */
 
@@ -72,7 +72,7 @@ static int httpd_debug = 0;
 #define SOAPCONN_STATE_INREQPAYLOAD 2
 #define SOAPCONN_STATE_PROCESSING   3
 
-/* 
+/*
  * In the PROCESSING state, we wouldn't generally need to be reading from
  * the client.  However, if the client closes their connection between the
  * time we stop reading the headers (or request payload) and the time we
@@ -87,7 +87,7 @@ static int reqmonitorbuf(struct nafmodule *mod, struct nafconn *conn)
 
 	if (!(nbuf = naf_malloc_type(mod, NAF_MEM_TYPE_NETBUF, SOAPMONBUFLEN)))
 		return -1;
-	if (naf_conn_reqread(conn, nbuf, SOAPMONBUFLEN, 0) == -1) {
+	if (naf_conn_reqread(conn, (naf_u8_t *)nbuf, SOAPMONBUFLEN, 0) == -1) {
 		naf_free(mod, nbuf);
 		return -1;
 	}
@@ -103,7 +103,7 @@ static int reqsoapbuf(struct nafmodule *mod, struct nafconn *conn, char *inbuf)
 		if (!(nbuf = naf_malloc_type(mod, NAF_MEM_TYPE_NETBUF, SOAPBUFLEN)))
 			return -1;
 	}
-	if (naf_conn_reqread(conn, inbuf ? inbuf : nbuf, SOAPBUFLEN, 0) == -1) {
+	if (naf_conn_reqread(conn, (naf_u8_t *)(inbuf ? inbuf : nbuf), SOAPBUFLEN, 0) == -1) {
 		naf_free(mod, nbuf);
 		return -1;
 	}
@@ -124,7 +124,7 @@ static int takeconn(struct nafmodule *mod, struct nafconn *conn)
 
 	conn->state = SOAPCONN_STATE_FRESH;
 
-	if ((naf_conn_setdelim(mod, conn, "\r\n", 2) == -1) ||
+	if ((naf_conn_setdelim(mod, conn, (naf_u8_t *)"\r\n", 2) == -1) ||
 			(reqsoapbuf(mod, conn, NULL) == -1)) {
 		return -1;
 	}
@@ -304,6 +304,20 @@ static struct httpdpage *naf_httpd_page__find_hard(struct nafmodule *mod, const
 	return NULL;
 }
 
+static struct httpdpage *naf_httpd_page__find_loose(struct nafmodule *mod, const char *fn)
+{
+	struct httpdpage *hp;
+
+	for (hp = naf_httpd__pages; hp; hp = hp->hp_next) {
+		if (hp->hp_flags & NAF_HTTPD_PAGEFLAGS_LOOSEMATCH) {
+			if (strncasecmp(fn, hp->hp_fn, strlen(hp->hp_fn)) == 0)
+				return hp;
+		}
+	}
+
+	return NULL;
+}
+
 int naf_httpd_page_register(struct nafmodule *theirmod, const char *fn, const char *contenttype, naf_httpd_pageflags_t flags, naf_httpd_pagehandler_t handler)
 {
 	struct httpdpage *hp;
@@ -342,6 +356,7 @@ int naf_httpd_page_unregister(struct nafmodule *theirmod, const char *fn)
 	return -1; /* not registered */
 }
 
+#ifndef NOXML
 int naf_httpd_sendlmx(struct nafmodule *theirmod, struct nafconn *conn, lmx_t *lmx)
 {
 	char *str, *str2;
@@ -363,6 +378,7 @@ int naf_httpd_sendlmx(struct nafmodule *theirmod, struct nafconn *conn, lmx_t *l
 
 	return 0;
 }
+#endif
 
 #define GENERIC404 { \
 	"HTTP/1.0 404 Not Found\r\n" \
@@ -387,7 +403,7 @@ static int send404(struct nafmodule *mod, struct nafconn *conn)
 	if (!(reply = naf_strdup_type(mod, NAF_MEM_TYPE_NETBUF, err404)))
 		return -1;
 
-	if (naf_conn_reqwrite(conn, reply, strlen(reply)) == -1) {
+	if (naf_conn_reqwrite(conn, (naf_u8_t *)reply, strlen(reply)) == -1) {
 		naf_free(mod, reply);
 		return -1;
 	}
@@ -397,23 +413,32 @@ static int send404(struct nafmodule *mod, struct nafconn *conn)
 	return 0;
 }
 
-static int dorequest_get(struct nafmodule *mod, struct nafconn *conn, const char *file)
+static int dorequest_get(struct nafmodule *mod, struct nafconn *conn, char *file)
 {
 	struct httpdpage *hp;
+	char *args = NULL;
+
+	if (strchr(file, '?')) {
+		args = strchr(file, '?');
+		*(args++) = '\0';
+	}
 
 	/* XXX if ends with /, fall back to looking for index.html, etc */
 	hp = naf_httpd_page__find_hard(mod, file);
 	if (!hp) {
-		if (httpd_debug > 0)
-			dvprintf(mod, "no registered handler for page '%s'\n", file);
-		return send404(mod, conn);
+		hp = naf_httpd_page__find_loose(mod, file);
+		if (!hp) {
+			if (httpd_debug > 0)
+				dvprintf(mod, "no handler matching page '%s'\n", file);
+			return send404(mod, conn);
+		}
 	}
 
 	if (!(hp->hp_flags & NAF_HTTPD_PAGEFLAGS_NOHEADERS)) {
 		char twoohoh[255];
 		char *hdr;
 
-		snprintf(twoohoh, sizeof(twoohoh), 
+		snprintf(twoohoh, sizeof(twoohoh),
 				"HTTP/1.0 200\r\n"
 				"Connection: Close\r\n"
 				"Content-Type: %s\r\n"
@@ -422,12 +447,12 @@ static int dorequest_get(struct nafmodule *mod, struct nafconn *conn, const char
 		if (!(hdr = naf_strdup_type(mod, NAF_MEM_TYPE_NETBUF, twoohoh)))
 			return -1;
 
-		if (naf_conn_reqwrite(conn, hdr, strlen(hdr)) == -1) {
+		if (naf_conn_reqwrite(conn, (naf_u8_t *)hdr, strlen(hdr)) == -1) {
 			naf_free(mod, hdr);
 			return -1;
 		}
 	}
-	hp->hp_handler(hp->hp_owner, hp->hp_fn, hp->hp_flags, conn);
+	hp->hp_handler(hp->hp_owner, file, hp->hp_flags, args, conn);
 
 	if (!(hp->hp_flags & NAF_HTTPD_PAGEFLAGS_NOAUTOCLOSE))
 		naf_conn_schedulekill(conn);
@@ -435,6 +460,8 @@ static int dorequest_get(struct nafmodule *mod, struct nafconn *conn, const char
 	return 0;
 }
 
+#ifndef NOXML
+
 struct handlenafrpcinfo {
 	struct nafmodule *mod;
 	XML_Parser parser;
@@ -611,7 +638,7 @@ static int addargs(struct nafmodule *mod, lmx_t *x, naf_rpc_arg_t *head)
 		 * into tag names here, but having XML tag names start with
 		 * numbers is technically illegal and makes many XML parsers
 		 * barf.  For example, the one used by SOAP::Lite.  Should
-		 * find a way around this.  
+		 * find a way around this.
 		 */
 		if (!(y = lmx_add(x, arg->name)))
 			continue;
@@ -793,12 +820,15 @@ errout:
 		naf_rpc_request_free(mod, req);
 	return -1;
 }
+#endif /* ndef NOXML */
 
 static int dorequest_post(struct nafmodule *mod, struct nafconn *conn, const char *file, unsigned char *reqpayload, int reqpayloadlen)
 {
 
+#ifndef NOXML
 	if (strcasecmp(file, "/nafrpc") == 0)
 		return handlenafrpc(mod, conn, reqpayload, reqpayloadlen);
+#endif
 
 	return send404(mod, conn);
 }
@@ -850,7 +880,7 @@ static int connready(struct nafmodule *mod, struct nafconn *conn, naf_u16_t what
 
 		if (conn->state == SOAPCONN_STATE_FRESH) {
 
-			if (handlefirstreqline(mod, conn, buf) == -1) {
+			if (handlefirstreqline(mod, conn, (char *)buf) == -1) {
 				naf_free(mod, buf);
 				/* XXX probably should issue HTTP error */
 				return -1;
@@ -858,7 +888,7 @@ static int connready(struct nafmodule *mod, struct nafconn *conn, naf_u16_t what
 
 			conn->state = SOAPCONN_STATE_INHEADERS;
 
-			if (reqsoapbuf(mod, conn, buf) == -1) {
+			if (reqsoapbuf(mod, conn, (char *)buf) == -1) {
 				naf_free(mod, buf);
 				return -1;
 			}
@@ -867,7 +897,7 @@ static int connready(struct nafmodule *mod, struct nafconn *conn, naf_u16_t what
 
 		} else if (conn->state == SOAPCONN_STATE_INHEADERS) {
 
-			if (strlen(buf) == 0) { /* headers end with blank line */
+			if (strlen((char *)buf) == 0) { /* headers end with blank line */
 				int clen = -1;
 
 				naf_conn_tag_fetch(mod, conn, "conn.httpheader.content-length", NULL, (void **)&clen);
@@ -900,12 +930,12 @@ static int connready(struct nafmodule *mod, struct nafconn *conn, naf_u16_t what
 
 			} else { /* header line */
 
-				if (handleheaderline(mod, conn, buf) == -1) {
+				if (handleheaderline(mod, conn, (char *)buf) == -1) {
 					naf_free(mod, buf);
 					return -1; /* XXX HTTP error */
 				}
 
-				if (reqsoapbuf(mod, conn, buf) == -1) {
+				if (reqsoapbuf(mod, conn, (char *)buf) == -1) {
 					naf_free(mod, buf);
 					return -1;
 				}
@@ -970,12 +1000,144 @@ static void freetag(struct nafmodule *mod, void *object, const char *tagname, ch
 	return;
 }
 
-static int modfirst(struct nafmodule *mod)
+static int
+sendbasicdata(struct nafmodule *mod, struct nafconn *conn, const char *format, ...)
+{
+	char *resp;
+	va_list ap;
+
+	/* XXX */
+	if (!(resp = naf_malloc_type(mod, NAF_MEM_TYPE_NETBUF, 1024)))
+		return -1;
+	va_start(ap, format);
+	vsnprintf(resp, 1024, format, ap);
+	va_end(ap);
+
+	if (naf_conn_reqwrite(conn, (naf_u8_t *)resp, strlen(resp)) == -1) {
+		naf_free(mod, resp);
+		return -1;
+	}
+	return 0;
+}
+
+static int
+nafrpc_page_handler(struct nafmodule *mod, const char *fn, naf_httpd_pageflags_t pageflags, const char *reqargs, struct nafconn *httpdconn)
+{
+	char module[64], method[64], *c, *c2;
+	naf_rpc_req_t *req;
+
+	fn += strlen("/nafrpc");
+	if ((fn[0] != '/') || !(c = strchr(fn + 1, '/'))) {
+		sendbasicdata(mod, httpdconn, "invalid url syntax (%s)", fn);
+		return 0;
+	}
+	c2 = c + strlen(c);
+	if (((c - fn) >= sizeof(module)) || ((c2 - c) >= sizeof(method))) {
+		sendbasicdata(mod, httpdconn, "invalid target specification (%s)", fn);
+		return 0;
+	}
+	memcpy(module, fn + 1, c - fn - 1); module[c - fn - 1] = '\0';
+	memcpy(method, c + 1, c2 - c - 1); method[c2 - c - 1] = '\0';
+
+	sendbasicdata(mod, httpdconn, "module='%s'<br/>method='%s'<br/>args='%s'<br/>", module, method, reqargs);
+
+	if (!(req = naf_rpc_request_new(mod, module, method))) {
+		sendbasicdata(mod, httpdconn, "<br/>unable to begin RPC request<br/>");
+		return 0;
+	}
+
+	while (reqargs && strlen(reqargs)) {
+		const char *name = reqargs;
+		char name2[128];
+		const char *value;
+		char value2[512];
+		const char *end;
+
+		if (!(end = strchr(name, '&')))
+			end = reqargs + strlen(reqargs);
+
+		if ((value = strchr(name, '=')) && (value < end))
+			value++;
+		else
+			value = end;
+
+		if ((value - reqargs) > sizeof(name2))
+			goto nextarg;
+		memcpy(name2, reqargs, value - reqargs);
+		name2[value - reqargs] = '\0';
+		if (name2[strlen(name2) - 1] == '=')
+			name2[strlen(name2) - 1] = '\0';
+		if ((end - value) > sizeof(value2))
+			goto nextarg;
+		memcpy(value2, value, end - value);
+		value2[end - value] = '\0';
+
+		sendbasicdata(mod, httpdconn, "argument: name='%s', value='%s'<br/>", name2, value2);
+
+nextarg:	/* go next argument */
+		if (strlen(reqargs))
+			reqargs++;
+		if ((reqargs = strchr(reqargs, '&')))
+			reqargs++;
+	}
+
+
+	if (naf_rpc_request_issue(mod, req) == -1) {
+		sendbasicdata(mod, httpdconn, "<br/>unable to issue RPC request<br/>");
+		naf_rpc_request_free(mod, req);
+		return 0;
+	}
+
+	{
+		const char *code = "Unknown response code";
+		if (req->status == NAF_RPC_STATUS_SUCCESS)
+			code = "Success";
+		else if (req->status == NAF_RPC_STATUS_UNKNOWNFAILURE)
+			code = "Unknown failure";
+		else if (req->status == NAF_RPC_STATUS_UNKNOWNTARGET)
+			code = "Unknown module";
+		else if (req->status == NAF_RPC_STATUS_UNKNOWNMETHOD)
+			code = "Unknown method";
+		else if (req->status == NAF_RPC_STATUS_PENDING)
+			code = "Request pending";
+
+		sendbasicdata(mod, httpdconn, "<br/><b>%s</b> (0x%04x)<br/>",
+			      code, req->status);
+
+		/* XXX send back return values */
+	}
+
+	naf_rpc_request_free(mod, req);
+
+	return 0;
+}
+
+static int modinit(struct nafmodule *mod)
 {
 
 	ourmodule = mod;
 
+	naf_httpd_page_register(mod, "/nafrpc", "text/html", NAF_HTTPD_PAGEFLAGS_LOOSEMATCH, nafrpc_page_handler);
+
+	return 0;
+}
+
+static int modshutdown(struct nafmodule *mod)
+{
+
+	naf_httpd_page_unregister(mod, "/nafrpc");
+
+	ourmodule = NULL;
+
+	return 0;
+}
+
+static int modfirst(struct nafmodule *mod)
+{
+
 	naf_module_setname(mod, "httpd");
+	mod->init = modinit;
+	mod->shutdown = modshutdown;
 	mod->takeconn = takeconn;
 	mod->connready = connready;
 	mod->connkill = connkill;
@@ -984,14 +1146,8 @@ static int modfirst(struct nafmodule *mod)
 	return 0;
 }
 
-#endif /* ndef NOXML */
-
 int naf_httpd__register(void)
 {
-#ifdef NOXML
-	return 0;
-#else
 	return naf_module__registerresident("httpd", modfirst, NAF_MODULE_PRI_THIRDPASS);
-#endif
 }
 
