@@ -54,7 +54,8 @@ int timps_oscar__debug = TIMPS_OSCAR_DEBUG_DEFAULT;
 struct nafmodule *timps_oscar__module = NULL;
 #define TIMPS_OSCAR_AUTHORIZER_DEFAULT "login.oscar.aol.com:5190"
 char *timps_oscar__authorizer = NULL;
-
+#define TIMPS_OSCAR_ENABLEPROROGUEALL_DEFAULT 0
+int timps_oscar__enableprorogueall = TIMPS_OSCAR_ENABLEPROROGUEALL_DEFAULT;
 
 static int
 toscar_msgrouting(struct nafmodule *mod, int stage, struct gnrmsg *gm, struct gnrmsg_handler_info *gmhi)
@@ -73,7 +74,7 @@ toscar_msgrouting(struct nafmodule *mod, int stage, struct gnrmsg *gm, struct gn
 	/* If the source is local and one of ours, take it. */
 	if ((gmhi->srcnode->metric == GNR_NODE_METRIC_LOCAL) &&
 					(gmhi->srcnode->ownermod == mod)) {
-		gm->routeflags |= GNR_MSG_ROUTEFLAG_ROUTED_FORWARD; 
+		gm->routeflags |= GNR_MSG_ROUTEFLAG_ROUTED_FORWARD;
 		return 1;
 	}
 
@@ -154,9 +155,11 @@ freetag(struct nafmodule *mod, void *object, const char *tagname, char tagtype,
 		; /* an int */
 	else if (strcmp(tagname, "conn.screenname") == 0) {
 		char *sn = (char *)tagdata;
+		struct nafconn *conn = (struct nafconn *)object;
 		struct gnrnode *node;
 
-		if ((node = gnr_node_findbyname(sn, OSCARSERVICE)))
+		if ((node = gnr_node_findbyname(sn, OSCARSERVICE)) &&
+				!toscar__userhasotherclient(mod, sn, conn))
 			gnr_node_offline(node, GNR_NODE_OFFLINE_REASON_DISCONNECTED);
 
 		naf_free(mod, sn);
@@ -258,6 +261,10 @@ signalhandler(struct nafmodule *mod, struct nafmodule *source, int signum)
 		NAFCONFIG_UPDATESTRMODPARMDEF(mod, "authorizer",
 					      timps_oscar__authorizer,
 					      TIMPS_OSCAR_AUTHORIZER_DEFAULT);
+
+		NAFCONFIG_UPDATEBOOLMODPARMDEF(mod, "enableprorogueall",
+					       timps_oscar__enableprorogueall,
+					       TIMPS_OSCAR_ENABLEPROROGUEALL_DEFAULT);
 	}
 
 	return;
